using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;

namespace SocietyGames.SpeechWorkout
{
    public class ExerciseData
    {
        private string name;        
        private float currentRanking;
        private float highestRanking;

        public ExerciseData(XmlNode node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            Read(node);
        }

        public ExerciseData(string name)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (name.Length == 0)
            {
                throw new ArgumentException("name");
            }
            this.name = name;            
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public float HighestRanking
        {
            get
            {
                return this.highestRanking;
            }
            set
            {
                this.highestRanking = value;
            }
        }

        public float CurrentRanking
        {
            get
            {
                return this.currentRanking;
            }
            set
            {
                this.currentRanking = value;
                if (this.currentRanking > this.highestRanking)
                {
                    this.highestRanking = this.currentRanking;
                }
            }
        }

        public void Write(XmlWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            writer.WriteStartElement("exercise");
            writer.WriteAttributeString("name", this.name);
            writer.WriteAttributeString("highestRanking", this.highestRanking.ToString());
            writer.WriteAttributeString("currentRanking", this.currentRanking.ToString());
            writer.WriteEndElement(); //data
        }

        public void Read(XmlNode node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            if (node.Name != "exercise")
            {
                throw new ArgumentException("node");
            }
            foreach (XmlAttribute attribute in node.Attributes)
            {
                switch (attribute.Name)
                {
                    case "name":
                        this.name = attribute.Value;
                        break;
                    case "highestRanking":
                        this.highestRanking = float.Parse(attribute.Value);
                        break;
                    case "currentRanking":
                        this.currentRanking = float.Parse(attribute.Value);
                        break;
                    default:
                        throw new FileFormatException("XML contained unknown element: " + attribute.Name);                        
                }
            }
        }
    }
}
