using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SocietyGames.SpeechWorkout
{
    public partial class ExerciseForm : Form
    {
        IExercise exercise;
        RankingChangedEventHandler correctlyAnsweredDelegate;
        RankingChangedEventHandler incorrectlyAnsweredDelegate;
        RankingChangedEventHandler taskCompletedDelegate;  

        int millisecondsLeft;
        int interval = 30000;
        float originalRanking;

        TimeSpan requestedRunTime;
        DateTime startTime;
        DateTime endTime;

        bool hasTimeLeft;

        public ExerciseForm()
        {
            InitializeComponent();
            this.correctlyAnsweredDelegate = new RankingChangedEventHandler(CorrectlyAnswered);
            this.incorrectlyAnsweredDelegate = new RankingChangedEventHandler(IncorrectlyAnswered);
            this.taskCompletedDelegate = new RankingChangedEventHandler(TaskCompleted);            
        }

        private void CorrectlyAnswered(object source, RankingChangedEventArgs eventArgs)
        {
            this.countLabel.Text = eventArgs.Ranking.ToString();
        }

        private void IncorrectlyAnswered(object source, RankingChangedEventArgs eventArgs)
        {
            this.countLabel.Text = eventArgs.Ranking.ToString();
        }

        private void TaskCompleted(object source, RankingChangedEventArgs eventArgs)
        {
            this.countLabel.Text = eventArgs.Ranking.ToString();
            if (this.hasTimeLeft)
            {
                //TODO: Show the start button.            
            }
            else
            {
                //TODO: Exercise Completed!
                MessageBox.Show("Well done! You've completed this exercise.");
            }
        }

        private void ShowExercise(IExercise exercise, float ranking, int millisecondsLeft)
        {
            if (exercise == null)
            {
                throw new ArgumentNullException("exercise");
            }
            if (ranking < 0)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }
            if (millisecondsLeft <= 0)
            {
                throw new ArgumentOutOfRangeException("minutes");
            }
            this.originalRanking = ranking;
            this.countLabel.Text = "0";
            this.millisecondsLeft = millisecondsLeft;
            this.progressBar.Minimum = 0;
            this.progressBar.Maximum = this.millisecondsLeft;
            this.progressBar.Step = this.interval;
            this.progressBar.Value = 0;
            SetExercise(exercise);
            this.exercise.Reset(ranking);
            this.hasTimeLeft = true;
            this.timer.Interval = this.interval;
            this.timer.Enabled = true;
            this.startTime = DateTime.Now;
            this.endTime = DateTime.Now;
            this.requestedRunTime = new TimeSpan(0, 0, 0, 0, millisecondsLeft);
        }

        public TimeSpan Time
        {
            get
            {
                TimeSpan time = this.endTime - this.startTime;
                if (time > this.requestedRunTime)
                {
                    time = this.requestedRunTime;
                }
                return time;
            }

        }

        private void SetExercise(IExercise exercise)
        {
            if (exercise == null)
            {
                throw new ArgumentNullException("exercise");
            }

            //Remove old
            if (this.exercise != null)
            {
                this.exercise.CorrectlyAnswered -= this.correctlyAnsweredDelegate;
                this.exercise.IncorrectlyAnswered -= this.incorrectlyAnsweredDelegate;
                this.exercise.TaskCompleted -= this.taskCompletedDelegate;                
                this.exercisePanel.Controls.Remove(this.exercise.Control);
            }
            //Do new
            this.exercise = exercise;
            this.exercise.CorrectlyAnswered += this.correctlyAnsweredDelegate;
            this.exercise.IncorrectlyAnswered += this.incorrectlyAnsweredDelegate;
            this.exercise.TaskCompleted += this.taskCompletedDelegate;                
            this.exercisePanel.Controls.Add(this.exercise.Control);
            this.exercise.Control.Dock = DockStyle.Fill;
            this.Text = "SpeechWorkout: " + this.exercise.Name;
        }

        private void timer_Tick(object sender, EventArgs e)
        {   
            //TODO: Get the actual time and compare! Don't assume the timer is running on minutes.
            this.millisecondsLeft -= this.timer.Interval;
            this.progressBar.PerformStep();
            if (this.millisecondsLeft <= 0)
            {
                this.hasTimeLeft = false;
                this.timer.Enabled = false;
            }            
        }

        private void ExerciseForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            this.endTime = DateTime.Now;
        }
    }
}