using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace SocietyGames.SpeechWorkout
{
    class ExerciseStats
    {
        private Dictionary<string, ExerciseData> exerciseDatas = new Dictionary<string, ExerciseData>();

        public ExerciseStats()
        {
        }

        /// <summary>
        /// Load from a node
        /// </summary>
        /// <param name="node"></param>
        public ExerciseStats(XmlNode node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            if (node.Name != "statistics")
            {
                throw new FileFormatException("Xml was expection \"statistics\" element.");
            }

            XmlNodeList children = node.ChildNodes;
            foreach (XmlNode childNode in children)
            {
                ExerciseData data = new ExerciseData(childNode);
                this.AddData(data);
            }
        }

        /// <summary>
        /// Write the data out.
        /// </summary>
        /// <param name="writer"></param>
        public void Write(XmlWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            writer.WriteStartElement("statistics");
            foreach (KeyValuePair<string, ExerciseData> pair in this.exerciseDatas)
            {
                pair.Value.Write(writer);
            }
            writer.WriteEndElement(); //statistics
        }

        public void AddData(ExerciseData data)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }
            this.exerciseDatas.Add(data.Name, data);
        }

        public ExerciseData GetData(string exerciseName)
        {
            if (exerciseName == null)
            {
                throw new ArgumentNullException("exerciseName");
            }
            return this.exerciseDatas[exerciseName];
        }

        public bool ContainsData(string exerciseName)
        {
            if (exerciseName == null)
            {
                throw new ArgumentNullException("exerciseName");
            }
            return this.exerciseDatas.ContainsKey(exerciseName);
        }
    }
}
