using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using System.Xml;

namespace SocietyGames.SpeechWorkout
{
    public partial class MainForm : Form
    {
        Control activeControl;
        LauncherControl launcherControl;
        FrameControl frameControl;
        readonly string title = "MemoryWorkout";
        readonly int exerciseDuration = 600000; //TODO: Fix this to be the real value.
        Record record;

        public MainForm()
        {
            InitializeComponent();
            //Try to load the XML.
            LoadRecord();
            //Create the launcher
            this.launcherControl = new LauncherControl(this.record);            
            this.launcherControl.LaunchExercise += new LaunchExerciseEventHandler(launcherControl_LaunchExercise);
            this.frameControl = new FrameControl();
            this.frameControl.ExerciseCompleted += new RankingChangedEventHandler(frameControl_ExerciseComplete);
            //Show the launcher.
            DisplayLauncherControl();            
        }

        #region Record

        private string GetRecordDirectory()
        {
            return Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "MemoryWorkout");
        }

        private string GetRecordFileName()
        {
            return Path.Combine(GetRecordDirectory(), "record.xml");
        }

        private void LoadRecord()
        {
            bool loaded = false;
            bool found = false;
            string fileName = GetRecordFileName();
            try
            {
                XmlDocument document = new XmlDocument();
                document.Load(fileName);
                this.record = new Record();
                this.record.Read(document.FirstChild);
                loaded = true;
            }
            catch (FileNotFoundException)
            {
                loaded = false;
            }
            catch (DirectoryNotFoundException)
            {
                loaded = false;
            }
            catch (XmlException)
            {
                loaded = false;
                found = true;
            }
            if (found && !loaded)
            {
                MessageBox.Show("The history file: " + fileName + " was not correclty formatted. The application will create a new one.");
            }
            if (this.record == null)
            {
                this.record = new Record();
            }
        }

        private void SaveRecord()
        {
            if (!Directory.Exists(GetRecordDirectory()))
            {
                Directory.CreateDirectory(GetRecordDirectory());                
            }
            string fileName = GetRecordFileName();            
            XmlTextWriter writer = new XmlTextWriter(fileName, Encoding.Unicode);
            writer.Formatting = Formatting.Indented;
            this.record.Write(writer);
            writer.Flush();
            writer.Close();            
        }
        #endregion

        void launcherControl_LaunchExercise(object source, LaunchExerciseEventArgs eventArgs)
        {
            this.Text = this.title + " : " + eventArgs.Exercise.Name;
            this.record.StartExercise(eventArgs.Exercise.Name);
            SetMainControl(this.frameControl);            
            this.frameControl.StartExercise(eventArgs.Exercise, eventArgs.Ranking, this.exerciseDuration);            
        }

        void frameControl_ExerciseComplete(object sender, RankingChangedEventArgs e)
        {
            this.Text = this.title;
            this.record.EndExercise(e.Ranking, new TimeSpan(0, 0, 0, 0, this.exerciseDuration));
            DisplayLauncherControl();
        }

        private void DisplayLauncherControl()
        {
            this.launcherControl.PrepareForDisplay();
            SetMainControl(this.launcherControl);
        }

        private void SetMainControl(Control control)
        {
            Debug.Assert(control != null, "Control cannot be null.");
            this.SuspendLayout();
            if (this.activeControl != null)
            {                
                this.Controls.Remove(this.activeControl);
            }            
            this.activeControl = control;
            this.activeControl.SuspendLayout();
            this.activeControl.Dock = DockStyle.Fill;
            this.Controls.Add(this.activeControl);
            this.activeControl.ResumeLayout();
            this.ResumeLayout();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                SaveRecord();
            }
            catch (Exception exception)
            {
                MessageBox.Show("Could not save record because: " + exception.Message);
                e.Cancel = true;
            }            
        }
    }
}
