using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using SpeechLib;
using System.Diagnostics;
using System.Resources;
using System.IO;
using System.Media;

//TODO: Block the UI when a previous sound is playing. Unblock when the voice finishes.
namespace SocietyGames.SpeechWorkout
{
    public struct RateAndSize
    {
        public Size Size;
        public int Rate;
        public int RatioWrongToRightToAdvance;
        public int RatioWrongToRightToRegress;
        public RateAndSize(int rate, int width, int height, int advanceRatio, int regressRatio)
        {
            if (width <= 0)
            {
                throw new ArgumentOutOfRangeException("width");
            }
            if (height <= 0)
            {
                throw new ArgumentOutOfRangeException("height");
            }
            if (advanceRatio <= 0)
            {
                throw new ArgumentOutOfRangeException("ratio");
            }
            this.Rate = rate;
            this.Size = new Size(width, height);
            this.RatioWrongToRightToAdvance = advanceRatio;
            this.RatioWrongToRightToRegress = regressRatio;
        }
    }

    public partial class MatchControl : ExerciseControl, IExercise
    {
        #region Variables
        float currentRanking;
        MatchButton[,] buttons;
        List<string> words;
        List<string> tempWords;        
        readonly int buttonWidth = 30;
        readonly int buttonHeight = 30;
        int columnCount;
        int rowCount;
        int activeButtonsCount;
        private SpVoice voice;
        Random random;
        string lastText;
        Point lastPoint;        
        List<string> syllables = new List<string>();
        List<string> currentSyllables = new List<string>();
        int clickCount;        
        private RateAndSize[] ratesAndSizes = new RateAndSize[]
        {
            new RateAndSize(-5, 4, 2, 3, 5),
            new RateAndSize(-5, 4, 4, 3, 5), 
            new RateAndSize(-5, 6, 4, 3, 5),
            new RateAndSize(-5, 6, 5, 3, 5),

            new RateAndSize(-4, 4, 2, 3, 5),
            new RateAndSize(-4, 4, 4, 3, 5), 
            new RateAndSize(-4, 6, 4, 3, 5),
            new RateAndSize(-4, 6, 5, 3, 5),

            new RateAndSize(-3, 4, 2, 3, 5),
            new RateAndSize(-3, 4, 4, 3, 5), 
            new RateAndSize(-3, 6, 4, 3, 5),
            new RateAndSize(-3, 6, 5, 3, 5),

            new RateAndSize(-2, 4, 2, 3, 5),
            new RateAndSize(-2, 4, 4, 3, 5), 
            new RateAndSize(-2, 6, 4, 3, 5),
            new RateAndSize(-2, 6, 5, 3, 5),

            new RateAndSize(-1, 4, 2, 3, 5),
            new RateAndSize(-1, 4, 4, 3, 5), 
            new RateAndSize(-1, 6, 4, 3, 5),
            new RateAndSize(-1, 6, 5, 3, 5),

            new RateAndSize(0, 4, 2, 3, 5),
            new RateAndSize(0, 4, 4, 3, 5), 
            new RateAndSize(0, 6, 4, 3, 5),
            new RateAndSize(0, 6, 5, 3, 5),

            new RateAndSize(1, 4, 2, 3, 5),
            new RateAndSize(1, 4, 4, 3, 5), 
            new RateAndSize(1, 6, 4, 3, 5),
            new RateAndSize(1, 6, 5, 3, 5),

            new RateAndSize(2, 4, 2, 3, 5),
            new RateAndSize(2, 4, 4, 3, 5), 
            new RateAndSize(2, 6, 4, 3, 5),
            new RateAndSize(2, 6, 5, 3, 5),

            new RateAndSize(3, 4, 2, 3, 5),
            new RateAndSize(3, 4, 4, 3, 5), 
            new RateAndSize(3, 6, 4, 3, 5),
            new RateAndSize(3, 6, 5, 3, 5),

            new RateAndSize(4, 4, 2, 3, 5),
            new RateAndSize(4, 4, 4, 3, 5), 
            new RateAndSize(4, 6, 4, 3, 5),
            new RateAndSize(4, 6, 5, 3, 5),

            new RateAndSize(5, 4, 2, 3, 5),
            new RateAndSize(5, 4, 4, 3, 5), 
            new RateAndSize(5, 6, 4, 3, 5),
            new RateAndSize(5, 6, 5, 3, 5),
        };
        #endregion

        public MatchControl()
        {
            InitializeComponent();
            this.voice = new SpVoice();
            this.random = new Random();
            this.words = new List<string>();
            this.tempWords = new List<string>();
            this.SuspendLayout();
            this.buttons = new MatchButton[this.ratesAndSizes[this.ratesAndSizes.Length - 1].Size.Width, this.ratesAndSizes[this.ratesAndSizes.Length - 1].Size.Height];
            for (int x = 0; x < this.buttons.GetLength(0); x++)
            {
                for (int y = 0; y < this.buttons.GetLength(1); y++)
                {
                    this.buttons[x, y] = new MatchButton();
                    this.buttons[x, y].FlatStyle = FlatStyle.System;
                    this.buttons[x, y].Enabled = false;
                    this.buttons[x, y].Visible = false;                    
                    this.buttons[x, y].Point = new Point(x, y);
                    this.buttons[x, y].Click += new EventHandler(Button_Click);
                    this.buttons[x, y].Width = this.buttonWidth;
                    this.buttons[x, y].Height = this.buttonHeight;
                    this.Controls.Add(this.buttons[x, y]);
                }
            }
            this.ResumeLayout(false);
            ResourceManager resourceManager = new ResourceManager(this.GetType());
            string fileText = (string)resourceManager.GetObject("Syllables");
            string[] items = fileText.Split((char[])null, StringSplitOptions.RemoveEmptyEntries);
            for (int on = 0; on < items.Length; on++)
            {
                this.syllables.Add(items[on]);
            }
        }

        private void LayoutButtons(int columnCount, int rowCount, int buttonWidth, int buttonHeight)
        {
            Debug.Assert(columnCount <= this.buttons.GetLength(0));
            Debug.Assert(rowCount <= this.buttons.GetLength(1));
            this.columnCount = columnCount;
            this.rowCount = rowCount;            
            this.SuspendLayout();
            int xOffset = (this.Width / 2) - ((columnCount * this.buttonWidth) / 2);
            int yOffset = (this.Height / 2) - ((rowCount * this.buttonHeight) / 2);
            for (int x = 0; x < columnCount; x++)
            {
                for (int y = 0; y < rowCount; y++)
                {
                    this.buttons[x, y].Left = this.buttonWidth * x + xOffset;
                    this.buttons[x, y].Top = this.buttonHeight * y + yOffset;
                    this.buttons[x, y].Visible = true;
                    this.buttons[x, y].Enabled = true;                    
                }
            }
            this.ResumeLayout(false);
            this.activeButtonsCount = this.columnCount * this.rowCount;
        }

        private void PopulateWords()
        {
            //Copy avaible words.
            this.currentSyllables.Clear();
            foreach (string syllable in this.syllables)
            {
                this.currentSyllables.Add(syllable);
            }
            //Now randomly assign them to our list.
            int wordCount = (this.columnCount * this.rowCount) / 2;
            Debug.Assert((this.columnCount * this.rowCount) / 2.0f == wordCount);
            for (int on = 0; on < wordCount; on++)
            {
                int randomValue = this.random.Next(0, this.currentSyllables.Count);
                this.words.Add(this.currentSyllables[randomValue]);
                this.words.Add(this.currentSyllables[randomValue]);
                this.tempWords.Add(this.currentSyllables[randomValue]);
                this.tempWords.Add(this.currentSyllables[randomValue]);
                this.currentSyllables.RemoveAt(randomValue);
            }
        }

        private void RandomlyAssignWords()
        {
            for (int x = 0; x < this.columnCount; x++)
            {
                for (int y = 0; y < this.rowCount; y++)
                {
                    int randomValue = this.random.Next(0, this.tempWords.Count);
                    //this.buttons[x, y].Text = this.tempWords[randomValue];
                    this.buttons[x, y].SpeechText = this.tempWords[randomValue];
                    this.tempWords.RemoveAt(randomValue);
                }
            }
        }

        void Button_Click(object sender, EventArgs e)
        {
            MatchButton button = (MatchButton)sender;
            this.voice.Speak(button.SpeechText, SpeechVoiceSpeakFlags.SVSFlagsAsync);
            this.clickCount++;
            //If they hit a matching pair
            if (button.SpeechText == this.lastText && this.lastPoint != button.Point)
            {                
                button.Enabled = false;
                button.Visible = false;
                this.buttons[this.lastPoint.X, this.lastPoint.Y].Enabled = false;
                this.buttons[this.lastPoint.X, this.lastPoint.Y].Visible = false;
                this.activeButtonsCount -= 2;
                //Last pair
                if (this.activeButtonsCount == 0)
                {
                    //Calculate the new ranking
                    ProcessAnswer();
                    //OnTaskCompleted will show the end of exercise UI, so we have to call correclty answered separately.
                    OnCorrectlyAnswered(this.currentRanking);
                    this.voice.Speak(string.Empty, SpeechVoiceSpeakFlags.SVSFPurgeBeforeSpeak); //Effectively end the speech.
                    OnTaskCompleted(this.currentRanking);
                }
                else
                {
                    OnCorrectlyAnswered(this.currentRanking);
                }
            }            
            this.lastText = button.SpeechText;
            this.lastPoint = button.Point;            
        }

        private void ProcessAnswer()
        {
            int buttonCount = this.columnCount * this.rowCount;
            if (this.clickCount <= buttonCount * RateAndSizeFromRanking(this.currentRanking).RatioWrongToRightToAdvance)
            {
                this.currentRanking += (1 / (float)this.ratesAndSizes.Length);
            }
            else if (this.clickCount >= buttonCount * RateAndSizeFromRanking(this.currentRanking).RatioWrongToRightToRegress)
            {
                this.currentRanking -= (1 / (float)this.ratesAndSizes.Length);
            }
            if (this.currentRanking > 1)
            {
                this.currentRanking = 1;
            }
            else if (this.currentRanking < 0)
            {
                this.currentRanking = 0;
            }
        }

        private void ShowNewBoard()
        {            
            RateAndSize rateAndSize = RateAndSizeFromRanking(this.currentRanking);
            Size size = rateAndSize.Size;
            this.voice.Rate = rateAndSize.Rate;
            LayoutButtons(size.Width, size.Height, this.buttonWidth, this.buttonHeight);
            PopulateWords();
            RandomlyAssignWords();            
        }

        private RateAndSize RateAndSizeFromRanking(float ranking)
        {
            if (ranking < 0 || ranking > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }
            int index = (int)(ranking * (float)this.ratesAndSizes.Length);
            if (index == this.ratesAndSizes.Length)
            {
                index = this.ratesAndSizes.Length - 1;
            }
            return this.ratesAndSizes[index];
        }

        #region IExercise Members

        string IExercise.Name
        {
            get
            {
                return "Match";
            }
        }

        public new void Reset(float ranking)
        {
            if (ranking < 0 || ranking > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }

            this.currentRanking = ranking;
        }

        public new void StartNextTask()
        {
            //Clear out any old state.
            this.lastText = String.Empty;
            this.lastPoint = new Point(-1, -1);
            this.clickCount = 0;
            
            ShowNewBoard();            
        }
        #endregion
    }

    public class MatchButton : Button
    {
        private string speechText;
        private Point point;

        public string SpeechText
        {
            get
            {
                return this.speechText;
            }
            set
            {
                this.speechText = value;
            }
        }

        public Point Point
        {
            get
            {
                return this.point;
            }
            set
            {
                this.point = value;
            }
        }
    }
}
