using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace SocietyGames.SpeechWorkout
{
    public class Record
    {
        bool isExerciseRunning = false;
        string currentExerciseName = String.Empty;
        Dictionary<string, ExerciseData> data = new Dictionary<string, ExerciseData>();
        TimeSpan totalTime;

        public bool Contains(string name)
        {
            return this.data.ContainsKey(name);
        }

        public ExerciseData GetData(string name)
        {
            return this.data[name];
        }

        public void StartExercise(string name)
        {
            if (name == null || name.Length == 0)
            {
                throw new ArgumentException("Argument cannot be null or empty.", "name");
            }
            this.currentExerciseName = name;
            this.isExerciseRunning = true;
        }
        
        public void EndExercise(float ranking, TimeSpan elapsedTime)
        {
            if (!this.isExerciseRunning)
            {
                throw new InvalidOperationException("EndExercise() called without matching StartExercise.");
            }
            ExerciseData exerciseData;
            if (!this.data.ContainsKey(this.currentExerciseName))
            {
                exerciseData = new ExerciseData(this.currentExerciseName);
                this.data.Add(this.currentExerciseName, exerciseData);
            }
            else
            {
                exerciseData = this.data[this.currentExerciseName];
            }
            exerciseData.CurrentRanking = ranking;            
            this.totalTime += elapsedTime;
            this.isExerciseRunning = false;
        }

        public string LastExerciseRun
        {
            get
            {
                return this.currentExerciseName;
            }
        }

        public void Write(XmlWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            writer.WriteStartElement("record");
            writer.WriteAttributeString("lastExercise", this.currentExerciseName);
            writer.WriteAttributeString("totalTime", this.totalTime.ToString());
            writer.WriteStartElement("exercises");
            foreach (KeyValuePair<string, ExerciseData> keyValuePair in this.data)
            {
                keyValuePair.Value.Write(writer);
            }
            writer.WriteEndElement(); //exercises
            writer.WriteEndElement(); //data
        }

        public void Read(XmlNode node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            if (node.Name != "record")
            {
                throw new ArgumentException("node");
            }
            foreach (XmlAttribute attribute in node.Attributes)
            {
                switch (attribute.Name)
                {
                    case "lastExercise":
                        this.currentExerciseName = attribute.Value;
                        break;
                    case "totalTime":
                        this.totalTime = TimeSpan.Parse(attribute.Value);
                        break;                    
                    default:
                        throw new FileFormatException("XML contained unknown element: " + attribute.Name);
                }
            }         
            foreach (XmlNode childNode in node.ChildNodes)
            {
                switch (childNode.Name)
                {
                    case "exercises":
                        ReadExercises(childNode);
                        break;
                }
            }
        }

        void ReadExercises(XmlNode node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            if (node.Name != "exercises")
            {
                throw new ArgumentException("node");
            }
            foreach (XmlNode childNode in node.ChildNodes)
            {
                switch (childNode.Name)
                {
                    case "exercise":
                        ExerciseData exerciseData = new ExerciseData(childNode);
                        this.data.Add(exerciseData.Name, exerciseData);
                        break;
                }
            }
        }
    }
}
