using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Media;
using System.Resources;
using SpeechLib;

namespace SocietyGames.SpeechWorkout
{
    public partial class RepeatControl : ExerciseControl, IExercise
    {
        float currentRanking;

        readonly char[] consonants = new char[] {'b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z'};
        readonly char[] vowels = new char[] { 'a', 'e', 'i', 'o', 'u' };

        private SpVoice voice;
        Random random = new Random();
        List<String> pressed = new List<String>();
        List<String> pattern = new List<String>();
        int onPattern;
        List<Button> buttons = new List<Button>();
        SoundPlayer correctSoundPlayer;
        SoundPlayer incorrectSoundPlayer;
        readonly float rankingAdjustmentRate = 0.01f;
        Staircase staircase = new Staircase(3, 1);

        readonly int buttonWidth = 75;
        readonly int buttonHeight = 23;
        readonly int buttonLeft = 38;

        public RepeatControl()
        {
            InitializeComponent();
            this.voice = new SpVoice();
            this.voice.EndStream += new _ISpeechVoiceEvents_EndStreamEventHandler(voice_EndStream);
            ResourceManager resourceManager = new ResourceManager(this.GetType());
            this.correctSoundPlayer = new SoundPlayer(resourceManager.GetStream("correct"));
            this.incorrectSoundPlayer = new SoundPlayer(resourceManager.GetStream("incorrect"));
        }

        private void CreateButtons(int count)
        {            
            if (count != this.buttons.Count)
            {
                this.SuspendLayout();
                //Remove the old buttons
                for (int on = 0; on < this.buttons.Count; on++)
                {
                    this.Controls.Remove(this.buttons[on]);
                    this.buttons[on].Dispose();
                }
                this.buttons.Clear();
                //Put in the new buttons
                for (int on = 0; on < count; on++)
                {
                    this.buttons.Add(new Button());
                    this.buttons[on].FlatStyle = FlatStyle.System;                    
                    this.buttons[on].Visible = false;
                    this.buttons[on].Width = this.buttonWidth;
                    this.buttons[on].Height = this.buttonHeight;
                    this.buttons[on].Left = this.buttonLeft;
                    this.buttons[on].Top = this.buttonHeight * on;
                    this.Controls.Add(this.buttons[on]);                    
                    this.buttons[on].Click += new EventHandler(Button_Click);
                }
                this.ResumeLayout();
            }
        }

        private void ProcessAnswer()
        {
            if (this.pattern.Count != this.pressed.Count)
            {
                throw new InvalidOperationException();
            }

            //Check if it's correct
            bool isCorrect = true;
            for (int onCheck = 0; onCheck < this.pattern.Count && isCorrect; onCheck++)
            {
                if (this.pattern[onCheck] != this.pressed[onCheck])
                {
                    isCorrect = false;
                }
            }
            //Adjust ranking
            int delta = this.staircase.Score(isCorrect);
            this.currentRanking += delta * this.rankingAdjustmentRate;
            if (this.currentRanking < 0)
            {
                this.currentRanking = 0;
            }
            if (this.currentRanking > 1)
            {
                this.currentRanking = 1;
            }
            //Indicate correctness.
            if (isCorrect)
            {
                OnCorrectlyAnswered(this.currentRanking);
            }
            else
            {
                OnIncorrectlyAnswered(this.currentRanking);
            }
            OnTaskCompleted(this.currentRanking);
        }

        /*private void Reset()
        {
            this.pattern.Clear();
            int syllableCount = (int)(this.currentRanking * 8) + 2; //2 - 9                
            for (int onSyllable = 0; onSyllable < syllableCount; onSyllable++)
            {
                this.pattern.Add(GenerateSyllable());
            }
            float remainder = (this.currentRanking % (1f / 8f));                
            this.voice.Rate = (int)(5 * (remainder * 8)); //0 - 5
            this.onPattern = 0;
            this.pressed.Clear();
            UpdateUIFromPattern();
            SetButtonsVisible(false);
            this.nextButton.Visible = true;
        }*/

        private string GenerateSyllable()
        {
            string syllable = String.Empty;
            syllable += this.consonants[this.random.Next(0, this.consonants.Length)];
            syllable += this.vowels[this.random.Next(0, this.vowels.Length)];
            syllable += this.consonants[this.random.Next(0, this.consonants.Length)];
            return syllable;
        }

        private void UpdateUIFromPattern()
        {
            CreateButtons(this.pattern.Count);
            List<int> indices = new List<int>(this.pattern.Count);
            for (int on = 0; on < this.pattern.Count; on++)
            {
                indices.Add(on);
            }
            for (int on = 0; on < this.pattern.Count; on++)
            {
                int randomIndex = this.random.Next(0, indices.Count);
                this.buttons[on].Text = this.pattern[indices[randomIndex]];                
                indices.RemoveAt(randomIndex);
            }
        }

        private void SetButtonsVisible(bool visible)
        {
            foreach (Button button in this.buttons)
            {
                button.Visible = visible;
            }
        }

        void Button_Click(object sender, EventArgs e)
        {
            this.pressed.Add(((Button)sender).Text);
            if (this.pressed.Count == this.pattern.Count)
            {
                SetButtonsVisible(false);
                ProcessAnswer();
            }
        }

        private void PlaySound()
        {
            this.pattern.Clear();
            int syllableCount = (int)(this.currentRanking * 8) + 2; //2 - 9                
            for (int onSyllable = 0; onSyllable < syllableCount; onSyllable++)
            {
                this.pattern.Add(GenerateSyllable());
            }
            float remainder = (this.currentRanking % (1f / 8f));
            this.voice.Rate = (int)(5 * (remainder * 8)); //0 - 5
            this.onPattern = 0;
            this.pressed.Clear();
            UpdateUIFromPattern();
            this.onPattern = 0;
            this.voice.Speak(this.pattern[this.onPattern], SpeechVoiceSpeakFlags.SVSFlagsAsync);
        }

        void voice_EndStream(int StreamNumber, object StreamPosition)
        {
            this.onPattern++;
            if (this.onPattern < this.pattern.Count)
            {
                this.voice.Speak(this.pattern[this.onPattern], SpeechVoiceSpeakFlags.SVSFlagsAsync);
            }
            else
            {
                SetButtonsVisible(true);
            }
        }

        #region IExercise Members

        string IExercise.Name
        {
            get { return "Repeat"; }
        }

        void IExercise.Reset(float ranking)
        {
            if (ranking < 0 || ranking > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }

            this.currentRanking = ranking;
            this.staircase.Clear();
            foreach (Button button in this.buttons)
            {
                button.Visible = false;
            }
        }

        void IExercise.StartNextTask()
        {            
            PlaySound();            
        }

        #endregion
    }
}
