using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace SocietyGames.SpeechWorkout
{
    public partial class TimerBar : UserControl
    {
        DateTime endTime;

        public TimerBar()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="totalTime">The total amount of time in milliseconds.</param>
        /// <param name="interval">Number of milliseconds between updates.</param>
        public void Start(int totalTime, int updateInterval)
        {
            if (totalTime < 0)
            {
                throw new ArgumentOutOfRangeException("totalTime");
            }
            if (updateInterval < 0)
            {
                throw new ArgumentOutOfRangeException("updateInterval");
            }
            if (updateInterval > totalTime)
            {
                throw new ArgumentOutOfRangeException("updateInterval", "Parameter must be less than or equal to totalTime.");
            }
            if (this.timer.Enabled)
            {
                throw new InvalidOperationException("Cannot Start() TimerBar that is already running.");
            }
            
            this.progressBar.Minimum = 0;
            this.progressBar.Maximum = totalTime;
            this.progressBar.Step = updateInterval;
            this.progressBar.Value = 0;

            this.timer.Interval = updateInterval;
            this.timer.Enabled = true;
                        
            this.endTime = DateTime.Now + new TimeSpan(0, 0, 0, 0, totalTime);            
        }

        public void Clear()
        {
            this.progressBar.Value = 0;
        }

        public void End()
        {
            this.timer.Enabled = false;
        }

        public bool IsRunning
        {
            get
            {
                return this.timer.Enabled;
            }
        }

        public event EventHandler TimeEnded;

        private void OnTimeEnded()
        {
            if (this.TimeEnded != null)
            {
                this.TimeEnded(this, EventArgs.Empty);
            }
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            this.progressBar.PerformStep();
            if (DateTime.Now >= this.endTime)
            {
                this.timer.Enabled = false;
                OnTimeEnded();
            }
        }
    }
}
