using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace SocietyGames.SpeechWorkout
{
    class Sound : IDisposable
    {
        SecondaryBuffer buffer;
        bool isDisposed = false;

        public Sound(Device device, IOscillator oscillator, float seconds)
        {
            if (device == null)
            {
                throw new ArgumentNullException("device");
            }
            if (oscillator == null)
            {
                throw new ArgumentNullException("oscillator");
            }
            if (seconds < 0)
            {
                throw new ArgumentOutOfRangeException("seconds");
            }

            WaveFormat waveFormat = new WaveFormat();
            waveFormat.SamplesPerSecond = 44100;
            waveFormat.Channels = 1;
            waveFormat.BitsPerSample = 16;
            waveFormat.AverageBytesPerSecond = waveFormat.SamplesPerSecond * waveFormat.Channels * (waveFormat.BitsPerSample / 8);
            waveFormat.BlockAlign = (short)((waveFormat.BitsPerSample / 8) * waveFormat.Channels);
            waveFormat.FormatTag = WaveFormatTag.Pcm;

            BufferDescription bufferDescription = new BufferDescription();
            bufferDescription.Flags = BufferDescriptionFlags.LocateInSoftware | BufferDescriptionFlags.GlobalFocus | BufferDescriptionFlags.ControlVolume;
            bufferDescription.BufferBytes = (int)(Math.Ceiling(waveFormat.AverageBytesPerSecond * seconds)); //TODO: Set this to whatever size
            bufferDescription.Format = waveFormat;

            byte[] data = new byte[bufferDescription.BufferBytes];            
            oscillator.Generate(data, bufferDescription.BufferBytes / 2);
            this.buffer = new SecondaryBuffer(bufferDescription, device);
            this.buffer.Write(0, data, LockFlag.None);
            this.buffer.Volume = -1000;
        }

        public void Play(int priority, BufferPlayFlags flags)
        {
            if (this.isDisposed)
            {
                throw new ObjectDisposedException("Sound");
            }
            this.buffer.Play(priority, flags);
        }

        #region IDisposable Members

        ~Sound()
        {
            Dispose(false);
        }

        protected virtual void Dispose(bool isDisposing)
        {
            if (this.isDisposed)
            {
                return;
            }
            if (isDisposing)
            {
                //Free managed resources
                if (this.buffer != null)
                {
                    this.buffer.Dispose();
                }
            }
            //Free unmanaged
            this.isDisposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
