using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Media;
using System.Resources;
using SpeechLib;

namespace SocietyGames.SpeechWorkout
{
    public partial class WhichOneControl : ExerciseControl, IExercise
    {
        #region Exercise Variables
        float currentRanking;        
        #endregion

        string answer;
        string userAnswer;
        Random random = new Random();
        Staircase staircase = new Staircase(3, 1);
        readonly float rankingAdjustmentRate = 0.01f;
        private SpVoice voice;
        List<SyllablePair> syllablePairs;

        struct SyllablePair
        {
            public readonly string Left;
            public readonly string Right;

            public SyllablePair(string left, string right)
            {
                this.Left = left;
                this.Right = right;
            }
        }

        public WhichOneControl()
        {
            InitializeComponent();
            this.voice = new SpVoice();
            this.voice.EndStream += new _ISpeechVoiceEvents_EndStreamEventHandler(voice_EndStream);
            this.syllablePairs = new List<SyllablePair>();
            try
            {
                using (System.IO.StreamReader file = new System.IO.StreamReader(".\\whichOne\\whichOne.txt"))
                {
                    string line;
                    char[] splitCharacters = new char[] {' '};
                    while ((line = file.ReadLine()) != null)
                    {
                        string[] lineParts = line.Split(splitCharacters, 3);
                        if (lineParts.Length != 2)
                        {
                            MessageBox.Show("WhichOne.txt had an invalid line: " + line);                            
                        }
                        this.syllablePairs.Add(new SyllablePair(lineParts[0], lineParts[1]));
                    }                    
                }
            }
            catch (System.IO.IOException)
            {
                MessageBox.Show("Could not find file whichOne.txt.");
            }
        }

        void voice_EndStream(int StreamNumber, object StreamPosition)
        {
            this.rightButton.Visible = true;
            this.leftButton.Visible = true;
        }

        private void ProcessAnswer()
        {
            if (this.userAnswer == null)
            {
                throw new InvalidOperationException("Called ResetOnPerformance before an answer was entered.");
            }

            bool isCorrect;
            if (this.userAnswer == this.answer)
            {
                isCorrect = true;                
            }
            else
            {
                isCorrect = false;                
            }
            int delta = this.staircase.Score(isCorrect);
            this.currentRanking += delta * this.rankingAdjustmentRate;
            if (this.currentRanking < 0)
            {
                this.currentRanking = 0;
            }
            if (this.currentRanking > 1)
            {
                this.currentRanking = 1;
            }
            if (isCorrect)
            {
                OnCorrectlyAnswered(this.currentRanking);
            }
            else
            {
                OnIncorrectlyAnswered(this.currentRanking);
            }
            this.leftButton.Visible = false;
            this.rightButton.Visible = false;
            OnTaskCompleted(this.currentRanking);
        }

        private void leftButton_Click(object sender, EventArgs e)
        {
            this.userAnswer = this.leftButton.Text;
            ProcessAnswer();
        }

        private void rightButton_Click(object sender, EventArgs e)
        {
            this.userAnswer = this.rightButton.Text;
            ProcessAnswer();
        }

        private void PlaySound()
        {
            this.voice.Rate = (int)(this.currentRanking * 10) - 5;
            SyllablePair pair = this.syllablePairs[this.random.Next(0, this.syllablePairs.Count)];
            this.leftButton.Text = pair.Left;
            this.rightButton.Text = pair.Right;
            if (this.random.NextDouble() < 0.5)
            {
                this.answer = this.leftButton.Text;
            }
            else
            {
                this.answer = this.rightButton.Text;
            }
            this.voice.Speak(this.answer, SpeechVoiceSpeakFlags.SVSFlagsAsync);
        }

        #region IExercise Members

        string IExercise.Name
        {
            get
            {
                return "Which One";
            }
        }
        
        public /*override*/ new void Reset(float rank)
        {
            if (rank < 0 || rank > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }
            this.currentRanking = rank;
            this.staircase.Clear();
            this.leftButton.Visible = false;
            this.rightButton.Visible = false;
        }

        public /*override*/ new void StartNextTask()
        {
            PlaySound();            
        }

        #endregion
    }
}
